/*
 * libmedia-thumbnail
 *
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact: Hyunjun Ko <zzoon.ko@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <glib.h>
#include <mm_file.h>
#include <mm_util_magick.h>
#include <media-util.h>
#include "media-thumbnail.h"
#include "media-thumb-debug.h"
#include "media-thumb-util.h"
#include "media-thumb-internal.h"
#include "media-thumb-ipc.h"

/* The maximum of resolution to be able to get thumbnail is 3000 x 3000, except for only jpeg */
#define THUMB_MAX_ALLOWED_MEM_FOR_THUMB 9000000
#define THUMB_MAX_ALLOWED_RESOLUTION 2000

int thumbnail_request_from_db_async(unsigned int request_id, const char *origin_path, ThumbFunc func, void *user_data, uid_t uid)
{
	int err = MS_MEDIA_ERR_NONE;
	ms_user_storage_type_e store_type = MS_USER_STORAGE_INTERNAL;

	thumb_retvm_if(!origin_path, MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid parameter");
	thumb_retvm_if(!g_file_test(origin_path, G_FILE_TEST_IS_REGULAR), MS_MEDIA_ERR_INVALID_PARAMETER, "No file [%s]", origin_path);

	err = ms_user_get_storage_type(uid, origin_path, &store_type);
	if ((err != MS_MEDIA_ERR_NONE) || ((store_type != MS_USER_STORAGE_INTERNAL) && (store_type != MS_USER_STORAGE_EXTERNAL))) {
		thumb_err_slog("origin path(%s) is invalid. err : [%d] store_type [%d]", origin_path, err, store_type);
		return MS_MEDIA_ERR_INVALID_PARAMETER;
	}

	thumb_dbg_slog("Path : %s", origin_path);

	thumbUserData *userData = (thumbUserData*)malloc(sizeof(thumbUserData));
	thumb_retvm_if(!userData, MS_MEDIA_ERR_OUT_OF_MEMORY, "Allocation failed");

	userData->func = (ThumbFunc)func;
	userData->user_data = user_data;

	/* Request for thumb file to the daemon "Thumbnail generator" */
	err = _media_thumb_request_async(THUMB_REQUEST_DB_INSERT, request_id, origin_path, userData, uid);
	if (err != MS_MEDIA_ERR_NONE) {
		thumb_err("_media_thumb_request failed : %d", err);
		SAFE_FREE(userData);
		return err;
	}

	return MS_MEDIA_ERR_NONE;
}

int thumbnail_request_extract_raw_data_async(int request_id, const char *origin_path, int width, int height, ThumbRawFunc func, void *user_data, uid_t uid)
{
	int err = MS_MEDIA_ERR_NONE;
	int exist = 0;

	thumb_retvm_if(!origin_path || request_id == 0, MS_MEDIA_ERR_INVALID_PARAMETER, "original path is NULL. Or there is an error in request_id.");

	/* check the file exits actually */
	exist = open(origin_path, O_RDONLY);
	if (exist < 0) {
		if (errno == EACCES || errno == EPERM) {
			thumb_err("Fail to open original_path[%s]: Permission Denied", origin_path);
			return  MS_MEDIA_ERR_PERMISSION_DENIED;
		} else {
			thumb_err("Fail to open original_path[%s]: Invalid Path", origin_path);
			return MS_MEDIA_ERR_INVALID_PARAMETER;
		}
	}
	close(exist);

	thumb_dbg_slog("Path : %s", origin_path);

	thumbRawUserData *userData = (thumbRawUserData*)malloc(sizeof(thumbRawUserData));
	thumb_retvm_if(!userData, MS_MEDIA_ERR_OUT_OF_MEMORY, "Allocation failed");

	userData->func = func;
	userData->user_data = user_data;

	err = _media_thumb_request_raw_data_async(THUMB_REQUEST_RAW_DATA, request_id, origin_path, width, height, userData, uid);
	if (err != MS_MEDIA_ERR_NONE) {
		thumb_err("_media_raw_thumb_request failed : %d", err);
		SAFE_FREE(userData);
		return err;
	}

	return MS_MEDIA_ERR_NONE;
}

int thumbnail_request_cancel_media(unsigned int request_id)
{
	int err = MS_MEDIA_ERR_NONE;

	err = _media_thumb_request_async(THUMB_REQUEST_CANCEL_MEDIA, request_id, NULL, NULL, 0);
	thumb_retvm_if(err != MS_MEDIA_ERR_NONE, err, "_media_thumb_request_async failed : %d", err);

	return err;
}

int thumbnail_request_cancel_raw_data(int request_id)
{
	int err = MS_MEDIA_ERR_NONE;

	thumb_retvm_if(request_id == 0, MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid parameter");

	err = _media_thumb_request_raw_data_async(THUMB_REQUEST_CANCEL_RAW_DATA, request_id, NULL, 0, 0, NULL, 0);
	thumb_retvm_if(err != MS_MEDIA_ERR_NONE, err, "_media_thumb_request_raw_data_async failed : %d", err);

	return err;
}

static void __get_rotation_and_cdis(const char *path, mm_util_magick_rotate_type *rot_type, int *cdis_value)
{
	int err = MS_MEDIA_ERR_NONE;
	MMHandleType tag = (MMHandleType) NULL;
	char *p = NULL;
	int size = 0;
	int _cdis_value = 0;
	mm_util_magick_rotate_type _rot_type = MM_UTIL_ROTATE_0;

	/* Get Content Tag attribute for orientation */
	err = mm_file_create_tag_attrs(&tag, path);
	if (err != FILEINFO_ERROR_NONE)
		return;

	err = mm_file_get_attrs(tag, MM_FILE_TAG_ROTATE, &p, &size, NULL);
	if (err == FILEINFO_ERROR_NONE && size >= 0 && p) {
		if (strncmp(p, "90", size) == 0)
			_rot_type = MM_UTIL_ROTATE_90;
		else if (strncmp(p, "180", size) == 0)
			_rot_type = MM_UTIL_ROTATE_180;
		else if (strncmp(p, "270", size) == 0)
			_rot_type = MM_UTIL_ROTATE_270;
		else
			_rot_type = MM_UTIL_ROTATE_0;

		thumb_dbg("There is tag rotate : %d", _rot_type);
	}

	err = mm_file_get_attrs(tag, MM_FILE_TAG_CDIS, &_cdis_value, NULL);
	if (err != FILEINFO_ERROR_NONE)
		_cdis_value = 0;

	*rot_type = _rot_type;
	*cdis_value = _cdis_value;

	mm_file_destroy_tag_attrs(tag);
}

static int __get_video_meta(int cdis_value, const char *path, int *video_track_num, unsigned int *width, unsigned int *height, void **frame, size_t *frame_size)
{
	int err = MS_MEDIA_ERR_NONE;
	MMHandleType content = (MMHandleType) NULL;
	int _video_track_num = 0;
	unsigned int _width = 0;
	unsigned int _height = 0;
	size_t _frame_size = 0;
	void *_frame = NULL;

	if (cdis_value == 1) {
		thumb_warn("This is CDIS vlaue 1");
		err = mm_file_create_content_attrs_safe(&content, path);
	} else {
		err = mm_file_create_content_attrs(&content, path);
	}
	thumb_retvm_if(err != FILEINFO_ERROR_NONE, MS_MEDIA_ERR_INTERNAL, "mm_file_create_content_attrs fails : %d", err);

	err = mm_file_get_attrs(content,
				MM_FILE_CONTENT_VIDEO_TRACK_COUNT, &_video_track_num,
				MM_FILE_CONTENT_VIDEO_WIDTH, &_width,
				MM_FILE_CONTENT_VIDEO_HEIGHT, &_height,
				MM_FILE_CONTENT_VIDEO_THUMBNAIL, &_frame, &_frame_size, /* raw image is RGB888 format */
				NULL);

	if (err != FILEINFO_ERROR_NONE) {
		thumb_err("mm_file_get_attrs fails : %d", err);
		mm_file_destroy_content_attrs(content);
		return MS_MEDIA_ERR_INTERNAL;
	}

	*video_track_num = _video_track_num;

	if (_video_track_num == 0) {
		mm_file_destroy_content_attrs(content);
		return MS_MEDIA_ERR_NONE;
	}

	if (!_frame || !_width || !_height) {
		thumb_err("wrong video info W[%d] H[%d] Size[%zu] Frame[%p]", _width, _height, _frame_size, _frame);
		mm_file_destroy_content_attrs(content);
		return MS_MEDIA_ERR_INTERNAL;
	}

	*width = _width;
	*height = _height;
	*frame_size = _frame_size;
	*frame = g_memdup(_frame, _frame_size);

	mm_file_destroy_content_attrs(content);

	return MS_MEDIA_ERR_NONE;
}


static int __get_video_info(const char *path, int *video_track_num, unsigned int *width, unsigned int *height, void **frame, size_t *frame_size, mm_util_magick_rotate_type *rot_type)
{
	int err = MS_MEDIA_ERR_NONE;
	int _cdis_value = 0;
	mm_util_magick_rotate_type _rot_type = MM_UTIL_ROTATE_0;

	__get_rotation_and_cdis(path, &_rot_type, &_cdis_value);
	err = __get_video_meta(_cdis_value, path, video_track_num, width, height, frame, frame_size);

	*rot_type = _rot_type;

	return err;
}

static void __media_thumb_get_proper_thumb_size(unsigned int origin_width, unsigned int origin_height, unsigned int *thumb_width, unsigned int *thumb_height)
{
	double ratio = 0.0;

	thumb_retm_if(origin_width == 0, "Invalid origin_width");
	thumb_retm_if(origin_height == 0, "Invalid origin_height");
	thumb_retm_if(!thumb_width, "Invalid thumb_width");
	thumb_retm_if(!thumb_height, "Invalid thumb_height");

	thumb_dbg("origin thumb w: %d h: %d", *thumb_width, *thumb_height);

	/* Set smaller length to default size */
	if (origin_width < origin_height) {
		if (origin_width < *thumb_width)
			*thumb_width = origin_width;
		ratio = (double)origin_height / (double)origin_width;
		*thumb_height = *thumb_width * ratio;
	} else {
		if (origin_height < *thumb_height)
			*thumb_height = origin_height;
		ratio = (double)origin_width / (double)origin_height;
		*thumb_width = *thumb_height * ratio;
	}

	thumb_dbg("proper thumb w: %d h: %d", *thumb_width, *thumb_height);
}

static int __get_video_thumb_to_file(unsigned int width, unsigned int height, void *frame, size_t frame_size, mm_util_magick_rotate_type rot_type, const char *thumb_path, unsigned int thumb_width, unsigned int thumb_height)
{
	int err = MS_MEDIA_ERR_NONE;
	mm_util_image_h img = NULL;
	mm_util_image_h resize_img = NULL;
	unsigned int thumb_w = thumb_width;
	unsigned int thumb_h = thumb_height;

	thumb_retvm_if(!thumb_path, MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid thumb_path");

	__media_thumb_get_proper_thumb_size(width, height, &thumb_w, &thumb_h);
	if (thumb_w == 0 || thumb_h == 0) {
		thumb_err("Failed to get thumb size");
		return MS_MEDIA_ERR_INTERNAL;
	}

	err = mm_image_create_image(width, height, MM_UTIL_COLOR_RGB24, (unsigned char *)frame, frame_size, &img);
	thumb_retvm_if(err != MM_UTIL_ERROR_NONE, err, "fail to mm_image_create_image [%d]", err);

	if (width > thumb_w || height > thumb_h) {
		if (rot_type != MM_UTIL_ROTATE_0) {
			err = mm_util_resize_B_B(img, thumb_w, thumb_h, &resize_img);
			if (err != MM_UTIL_ERROR_NONE)
				goto ERROR;

			err = mm_util_rotate_B_P(resize_img, rot_type, thumb_path);

		} else {
			err = mm_util_resize_B_P(img, thumb_w, thumb_h, thumb_path);
		}
	} else {
		if (rot_type != MM_UTIL_ROTATE_0)
			err = mm_util_rotate_B_P(img, rot_type, thumb_path);
		else
			err = mm_util_resize_B_P(img, width, height, thumb_path);
	}

ERROR:
	mm_image_destroy_image(img);
	mm_image_destroy_image(resize_img);
	if (err == MS_MEDIA_ERR_NONE)
		return err;

	return MS_MEDIA_ERR_INTERNAL;
}

static int __get_video_thumb_to_buffer(unsigned int width, unsigned int height, void *frame, size_t frame_size, mm_util_magick_rotate_type rot_type, unsigned int thumb_width, unsigned int thumb_height, mm_util_image_h *dst_img)
{
	int err = MS_MEDIA_ERR_NONE;
	mm_util_image_h img = NULL;
	mm_util_image_h resize_img = NULL;
	unsigned int thumb_w = thumb_width;
	unsigned int thumb_h = thumb_height;

	__media_thumb_get_proper_thumb_size(width, height, &thumb_w, &thumb_h);
	if (thumb_w == 0 || thumb_h == 0) {
		thumb_err("Failed to get thumb size");
		return MS_MEDIA_ERR_INTERNAL;
	}

	err = mm_image_create_image(width, height, MM_UTIL_COLOR_RGB24, (unsigned char *)frame, frame_size, &img);
	thumb_retvm_if(err != MM_UTIL_ERROR_NONE, err, "fail to mm_image_create_image [%d]", err);

	if (width > thumb_w || height > thumb_h) {
		if (rot_type != MM_UTIL_ROTATE_0) {
			err = mm_util_resize_B_B(img, thumb_w, thumb_h, &resize_img);
			if (err != MM_UTIL_ERROR_NONE)
				goto ERROR;

			err = mm_util_rotate_B_B(resize_img, rot_type, dst_img);

		} else {
			err = mm_util_resize_B_B(img, thumb_w, thumb_h, dst_img);
		}
	} else {
		if (rot_type != MM_UTIL_ROTATE_0)
			err = mm_util_rotate_B_B(img, rot_type, dst_img);
		else
			err = mm_image_clone_image(img, dst_img);
	}

ERROR:
	mm_image_destroy_image(img);
	mm_image_destroy_image(resize_img);
	if (err != MS_MEDIA_ERR_NONE)
		return MS_MEDIA_ERR_INTERNAL;

	return MS_MEDIA_ERR_NONE;
}

static int __check_path_validity(const char *path)
{
	thumb_retvm_if(!path, MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid path");

	if (access(path, R_OK) < 0) {
		if (errno == EACCES || errno == EPERM) {
			thumb_err("Fail to open path: Permission Denied [%s]", path);
			return  MS_MEDIA_ERR_PERMISSION_DENIED;
		} else {
			thumb_err("Fail to open path: Invalid Path [%s]", path);
			return MS_MEDIA_ERR_INVALID_PARAMETER;
		}
	}

	return MS_MEDIA_ERR_NONE;
}

static int __check_thumb_path_validity(const char *path)
{
	char *dir_name = NULL;
	int ret = MS_MEDIA_ERR_NONE;

	thumb_retvm_if(!path, MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid path");

	dir_name = g_path_get_dirname(path);

	if (access(dir_name, W_OK) != 0) {
		if (errno == EACCES || errno == EPERM) {
			thumb_err("No permission to write[%s]", dir_name);
			ret = MS_MEDIA_ERR_PERMISSION_DENIED;
		} else {
			thumb_err("Does not exists[%s]", dir_name);
			ret = MS_MEDIA_ERR_INVALID_PARAMETER;
		}
	}

	SAFE_FREE(dir_name);

	return ret;
}

static int __check_parameter_validity_for_file(const char *path, unsigned int width, unsigned int height, const char *thumb_path)
{
	int err = MS_MEDIA_ERR_NONE;

	thumb_retvm_if((width > THUMB_MAX_ALLOWED_RESOLUTION || width == 0), MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid width[%d]", width);
	thumb_retvm_if((height > THUMB_MAX_ALLOWED_RESOLUTION || height == 0), MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid height[%d]", height);

	/* Check path is accessible */
	err = __check_path_validity(path);
	thumb_retvm_if(err != MS_MEDIA_ERR_NONE, err, "Invalid path");

	/* Check thumbnail path is writable */
	err = __check_thumb_path_validity(thumb_path);
	thumb_retvm_if(err != MS_MEDIA_ERR_NONE, err, "Invalid thumb_path");

	return MS_MEDIA_ERR_NONE;
}

static int __check_parameter_validity_for_buffer(const char *path, unsigned int width, unsigned int height, unsigned char **thumb_buffer, size_t *thumb_size, unsigned int *thumb_width, unsigned int *thumb_height)
{
	thumb_retvm_if((width > THUMB_MAX_ALLOWED_RESOLUTION || width == 0), MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid width[%d]", width);
	thumb_retvm_if((height > THUMB_MAX_ALLOWED_RESOLUTION || height == 0), MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid height[%d]", height);
	thumb_retvm_if(!thumb_buffer || !thumb_size || !thumb_width || !thumb_height, MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid out param");

	//Check path is accessible
	return __check_path_validity(path);
}

int create_video_thumbnail_to_file(const char *path, unsigned int width, unsigned int height, const char *thumb_path, bool auto_rotate)
{
	int err = MS_MEDIA_ERR_NONE;
	int video_track_num = 0;
	unsigned int video_width = 0;
	unsigned int video_height = 0;
	void *frame = NULL;
	size_t frame_size = 0;
	mm_util_magick_rotate_type rot_type = MM_UTIL_ROTATE_NUM;

	err = __check_parameter_validity_for_file(path, width, height, thumb_path);
	thumb_retvm_if(err != MS_MEDIA_ERR_NONE, err, "Invalid parameter");

	//Get video info
	err = __get_video_info(path, &video_track_num, &video_width, &video_height, &frame, &frame_size, &rot_type);
	thumb_retvm_if(err != MM_UTIL_ERROR_NONE, err, "fail to __get_video_info [%d]", err);
	thumb_retvm_if(video_track_num == 0, MM_UTIL_ERROR_NONE, "No video track");

	if (!auto_rotate)
		rot_type = MM_UTIL_ROTATE_0;

	//Extract thumbnail
	err = __get_video_thumb_to_file(video_width, video_height, frame, frame_size, rot_type, thumb_path, width, height);
	g_free(frame);

	return err;
}

int create_video_thumbnail_to_buffer(const char *path, unsigned int width, unsigned int height, unsigned char **thumb_buffer, size_t *thumb_size, unsigned int *thumb_width, unsigned int *thumb_height, bool auto_rotate)
{
	int err = MS_MEDIA_ERR_NONE;
	int video_track_num = 0;
	unsigned int video_w = 0;
	unsigned int video_h = 0;
	void *frame = NULL;
	size_t frame_size = 0;
	mm_util_image_h img = NULL;
	mm_util_magick_rotate_type rot_type = MM_UTIL_ROTATE_NUM;

	err = __check_parameter_validity_for_buffer(path, width, height, thumb_buffer, thumb_size, thumb_width, thumb_height);
	thumb_retvm_if(err != MS_MEDIA_ERR_NONE, err, "Invalid parameter");

	//Get video info
	err = __get_video_info(path, &video_track_num, &video_w, &video_h, &frame, &frame_size, &rot_type);
	thumb_retvm_if(err != MM_UTIL_ERROR_NONE, err, "fail to __get_video_info [%d]", err);
	thumb_retvm_if(video_track_num == 0, MM_UTIL_ERROR_NONE, "No video track");

	if (!auto_rotate)
		rot_type = MM_UTIL_ROTATE_0;

	//Extract thumbnail
	err = __get_video_thumb_to_buffer(video_w, video_h, frame, frame_size, rot_type, width, height, &img);
	if (err == MS_MEDIA_ERR_NONE)
		err = mm_image_get_image(img, thumb_width, thumb_height, NULL, thumb_buffer, thumb_size);

	mm_image_destroy_image(img);
	g_free(frame);

	return err;
}

static int __get_image_info(const char *path, unsigned int *width, unsigned int *height)
{
	int err = MS_MEDIA_ERR_NONE;
	mm_util_img_codec_type image_type = 0;
	unsigned int image_w = 0;
	unsigned int image_h = 0;

	thumb_retvm_if(!path, MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid path");
	thumb_retvm_if(!width, MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid width");
	thumb_retvm_if(!height, MS_MEDIA_ERR_INVALID_PARAMETER, "Invalid height");

	err = mm_util_extract_image_info(path, &image_type, &image_w, &image_h);
	thumb_retvm_if(err != MS_MEDIA_ERR_NONE, MS_MEDIA_ERR_INTERNAL, "Getting image info is failed err: %d", err);
	thumb_retvm_if(image_type == IMG_CODEC_UNKNOWN_TYPE, MS_MEDIA_ERR_UNSUPPORTED_CONTENT, "Unsupported image codec");

	/*
	if ((image_type != IMG_CODEC_JPEG) && (image_w * image_h > THUMB_MAX_ALLOWED_MEM_FOR_THUMB)) {
		thumb_warn("This original image is too big. w[%d] h[%d] size limit[%d]", image_w, image_h, THUMB_MAX_ALLOWED_MEM_FOR_THUMB);
		return MS_MEDIA_ERR_THUMB_TOO_BIG;
	}
	*/

	*width = image_w;
	*height = image_h;

	return MS_MEDIA_ERR_NONE;
}

int create_image_thumbnail_to_file(const char *path, unsigned int width, unsigned int height, const char *thumb_path)
{
	int err = MS_MEDIA_ERR_NONE;
	unsigned int image_w = 0;
	unsigned int image_h = 0;
	unsigned int thumb_w = width;
	unsigned int thumb_h = height;

	err = __check_parameter_validity_for_file(path, width, height, thumb_path);
	thumb_retvm_if(err != MS_MEDIA_ERR_NONE, err, "Invalid parameter");

	//Get image info
	err = __get_image_info(path, &image_w, &image_h);
	thumb_retvm_if(err != MM_UTIL_ERROR_NONE, err, "fail to __get_image_info [%d]", err);

	//Extract thumbnail
	__media_thumb_get_proper_thumb_size(image_w, image_h, &thumb_w, &thumb_h);

	err = mm_util_resize_P_P(path, thumb_w, thumb_h, thumb_path);
	thumb_retvm_if(err != MM_UTIL_ERROR_NONE, MS_MEDIA_ERR_INTERNAL, "mm_util_resize_P_P failed : %d", err);

	return MS_MEDIA_ERR_NONE;
}

int create_image_thumbnail_to_buffer(const char *path, unsigned int width, unsigned int height, unsigned char **thumb_buffer, size_t *thumb_size, unsigned int *thumb_width, unsigned int *thumb_height)
{
	int err = MS_MEDIA_ERR_NONE;
	unsigned int image_w = 0;
	unsigned int image_h = 0;
	unsigned int thumb_w = width;
	unsigned int thumb_h = height;
	mm_util_image_h img = NULL;

	err = __check_parameter_validity_for_buffer(path, width, height, thumb_buffer, thumb_size, thumb_width, thumb_height);
	thumb_retvm_if(err != MS_MEDIA_ERR_NONE, err, "Invalid parameter");

	//Get image info
	err = __get_image_info(path, &image_w, &image_h);
	thumb_retvm_if(err != MM_UTIL_ERROR_NONE, err, "fail to __get_image_info [%d]", err);

	//Extract thumbnail
	__media_thumb_get_proper_thumb_size(image_w, image_h, &thumb_w, &thumb_h);

	err = mm_util_resize_P_B(path, thumb_w, thumb_h, MM_UTIL_COLOR_BGRA, &img);
	thumb_retvm_if(err != MM_UTIL_ERROR_NONE, MS_MEDIA_ERR_INTERNAL, "mm_util_resize_P_B failed : %d", err);

	err = mm_image_get_image(img, thumb_width, thumb_height, NULL, thumb_buffer, thumb_size);

	mm_image_destroy_image(img);

	return err;
}
